
library(data.table)
library(caret)
library(doParallel)
library(ROCR)
library(DMwR)

# load dataset
tt<-fread("Rapid progressor GSE44216.csv",data.table=F)

# change names of classes
cl<-as.factor(ifelse(tt$class=="RP","Y","N"))

# remove column with classes from tt table
tt<-tt[,2:ncol(tt)]

# create K-fold indices
fold <- createFolds(cl, k = nrow(tt), list = FALSE)

# create empty vector for predicted probabilities
pred<-numeric(length(fold))

# create Progress Bar object
pb <- txtProgressBar(min = 0, max = nrow(tt), style = 3)

# set model hyperparameters for unbalanced datasets
trControl <- trainControl(method="none",sampling="smote")

# set model hyperparameters for balanced datasets
trControl <- trainControl(method="none")

# perform leave-one-out cross-validation
for(i in 1:nrow(tt)) {
  
  # create training and test sets
  tr.x<-tt[fold!=i,]
  test.x<-tt[fold==i,]
  tr.y<-cl[fold!=i]
  test.y<-cl[fold==i]
  
  # remove 50 % genes with lowest expression variance
  vr<-apply(tr.x,2,var)
  q<-quantile(vr,probs = 0.5)
  tr.x<-tr.x[,vr>q]
  test.x<-test.x[,vr>q]
  
  # remove highly correlated genes
  cr<-findCorrelation(cor(tr.x),cutoff = 0.9)
  tr.x<-tr.x[,-cr]
  test.x<-test.x[,-cr]
  
  # create model using Random Forest
  fit <- train(x=data.matrix(tr.x), y=tr.y,
                method="rf",
                trControl=trControl)
  
  # perform prediction for test set
  p<-predict(fit,newdata = test.x,type="prob")[,2]
  pred[fold==i]<-p
  
  setTxtProgressBar(pb, i)
}

close(pb) # delete Progress Bar object

# calculate area under the ROC curve (AUC)
auc<-prediction(pred,cl)
perf<-performance(auc,"auc")
perf@y.values[[1]]

# calculate accuracy values
pred.class<-as.factor(ifelse(pred>0.5,"Y","N"))
confusionMatrix(data=pred.class,reference=cl,positive="Y")
