
library(data.table)
library(caret)
library(doParallel)
library(ROCR)
library(HandTill2001)
library(DMwR)

# load dataset
tt<-fread("Elite controllers GSE87620.csv",data.table=F)

# change names of classes
cl<-as.factor(ifelse(tt$class==1,"Y1",ifelse(tt$class=="2","Y2","Y3")))

# Y1 - elite controllers whose CD8 T cells have transcription profiles similar to those from cART-treated patients
# Y2 - elite controllers whose CD8 T cells have unique transcription profiles
# Y3 - elite controllers whose CD8 T cells have transcription profiles similar to those from healthy people

# remove column with classes from tt table
tt<-tt[,2:ncol(tt)]

# create K-fold indices
fold <- createFolds(cl, k = nrow(tt), list = FALSE)

# create empty variable for predicted probabilities
pred<-NULL

# create Progress Bar object
pb <- txtProgressBar(min = 0, max = nrow(tt), style = 3)

# perform leave-one-out cross-validation
for(i in 1:nrow(tt)) {
  
  # create training and test sets
  tr.x<-tt[fold!=i,]
  test.x<-tt[fold==i,]
  tr.y<-cl[fold!=i]
  test.y<-cl[fold==i]
  
  # remove 50 % genes with lowest expression variance
  vr<-apply(tr.x,2,var)
  q<-quantile(vr,probs = 0.5)
  tr.x<-tr.x[,vr>q]
  test.x<-test.x[,vr>q]
  
  # remove highly correlated genes
  cr<-findCorrelation(cor(tr.x),cutoff = 0.9)
  tr.x<-tr.x[,-cr]
  test.x<-test.x[,-cr]
  
  # model hyperparameters for unbalanced dataset
  trControl <- trainControl(method="none",sampling="smote")
  
  # create model using Random Forest
  fit <- train(x=data.matrix(tr.x), y=tr.y,
                method="rf",
                trControl=trControl)
  
  # perform prediction for test set
  p<-predict(fit,newdata = test.x,type="prob")
  pred<-rbind(pred,p)
  
  setTxtProgressBar(pb, i)
}

close(pb) # delete Progress Bar object

# calculate average area under the ROC curve (AUC)
auc(multcap(
  response = cl,
  predicted = as.matrix(pred)
))

# calculate accuracy values
pred.class<-apply(pred,1,which.max)
pred.class<-as.factor(ifelse(pred.class==1,"Y1",ifelse(pred.class=="2","Y2","Y3")))
confusionMatrix(data=pred.class,reference=cl)

